{-# LANGUAGE NamedFieldPuns #-}
{-# LANGUAGE RecordWildCards #-}

module Shift
  ( shiftMain,
    module X,
  )
where

import Control.Lens ((^.))
import Options.Applicative
  ( execParser,
    fullDesc,
    header,
    helper,
    info,
    progDesc,
  )
import Shift.CLI as X
  ( HostingType (..),
    ShiftCommand (..),
    ShiftOptions (..),
    hostingType,
    shiftCommand,
    shiftOptions,
    soCommand,
    soFromRef,
    soGitHubOwner,
    soGitHubRepository,
    soGitHubToken,
    soHostingType,
    soIncludeHead,
    soToRef,
  )
import Shift.Git as X
  ( commitSummary,
    lookupCommitsDiff,
    lookupRawRef,
    pairedTagsFromOpts,
    parseAndSortRefs,
    parseTag,
    refsPairsFromRefs,
    renderDiff,
    renderToNode,
  )
import Shift.Rendering as X
  ( bold,
    headerOne,
    headerThree,
    headerTwo,
    indented,
    indentedL,
    leafNode,
    listItemNode,
    paragraphNode,
    parentNode,
    printReport,
    renderAuthor,
    renderConventionalCommit,
    renderMergeCommit,
    renderMiscCommit,
    renderRange,
    renderRef,
    renderTagRef,
    textNode,
  )
import Shift.Server (runServer)
import Shift.Types as X
  ( BreakingChange (..),
    ChangeReport (..),
    ClientState (..),
    CommitType (..),
    ConventionalCommit (..),
    ConventionalGroup,
    GitClientState (..),
    GitHubClientState (..),
    GitM,
    HasRepositoryCache (..),
    MergeCommit (..),
    MergeGroup,
    MiscCommit (..),
    MiscGroup,
    ParsedCommit (..),
    ParsedGroup,
    RepositoryCache (..),
    ShiftException (..),
    TagRef (..),
    TicketChange,
    bcBody,
    bcSubject,
    ccAffectedTickets,
    ccBody,
    ccBreakingChanges,
    ccScope,
    ccSubject,
    ccType,
    crAffectedTickets,
    crBreakingChanges,
    crChores,
    crDocs,
    crFeatures,
    crFixes,
    crMerges,
    crMisc,
    crRefactors,
    crStyles,
    crTests,
    executeRequest_,
    gcsAuth,
    gcsCache,
    gcsManager,
    gcsOwner,
    gcsRepository,
    lookupUserOnGitHub,
    lookupUserOnGitHubCommit,
  )
import Shift.Generate (runGenerate)

-- | The main CLI entrypoint.
shiftMain :: IO ()
shiftMain = do
  currentOptions <- execParser opts

  case currentOptions ^. soCommand of
    GenerateCommand {_gcOutputPath, ..} -> runGenerate _gcOutputPath currentOptions
    ServeCommand {_scPort, ..} -> runServer _scPort currentOptions
  where
    opts =
      info
        (helper <*> shiftOptions)
        ( fullDesc
            <> progDesc "Execute the given COMMAND"
            <> header "shift - A change log generator"
        )
