{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE ScopedTypeVariables #-}

module Shift.Server (runServer) where

import CMarkGFM (nodeToCommonmark, nodeToHtml)
import Control.Monad.IO.Class (MonadIO (liftIO))
import Data.Maybe (fromMaybe)
import Data.Text.Lazy (fromChunks)
import Shift.CLI (ShiftOptions)
import Shift.Git (renderToNode)
import Web.Scotty (get, html, param, rescue, scotty, text)

runServer :: Maybe Int -> ShiftOptions -> IO ()
runServer port options = scotty (fromMaybe 3000 port) $
  get "/" $ do
    (asMarkdown :: Bool) <- param "as_markdown" `rescue` (\_ -> pure False)

    node <- liftIO $ renderToNode options

    if asMarkdown
      then text $ fromChunks [nodeToCommonmark [] Nothing node]
      else html $ fromChunks [nodeToHtml [] [] node]
